;************************* DUMMY **********************
;Author...: Jerry Gilmore
;Date.....: 07-16-1988
;BBS......: (603) 352-7919
;Assembler: MASM 4.0
;Interface: CLIPPER Summer '87
;Notice...: Placed in the public domain 1988
;
;The folks at Nantucket decided that you can't write intelligent code.
;So they felt it necessary to link in everything for you so they can't
;be blamed when your program doesn't work. Well, I played around a bit and
;discovered that minimal programs could be written with more than half the
;library missing!
;
;I've gotten many responses from this SHRINKCL file. Thanks to all of you
;who dialed up! Some more explanations are in order:
;(1) I don't plan to market this as a product because I didn't write Clipper
;    so I can't possibly understand how all the entry points are used and
;    your programs might seem to work fine, except in some rare cases where
;    the code that was ripped out is now gone. No way can I support this with
;    out full knowledge of Clipper. But if I did market this, how would 
;    "CLIPPER LIGHT" sound? With 1/3rd the memory of regular Clipper.
;(2) I plan to fool around some more with this file and will update the file
;    some time.
;(3) What good is a Clipper program without file usage? Not very good, but
;    the intent is to simply show you how you can make "Clipper Light".
;(4) Some of you have asked why Nantucket links the whole library in. The best
;    explanation I can come up with is that Clipper, unlike Nantucket claims,
;    is not really a native mode compiler (semantic difference I suppose).
;    I believe that Clipper is closer to a p(seudo)-code interpreter. The
;    difference being that unlike Foxbase, Clipper's run time (engine) is
;    linked into your application code. I've done some things to Clipper
;    that isn't nice, but I've discovered that the compiler doesn't
;    generate op-codes or assembly language instructions like a C compiler
;    or BASIC compiler would. So my intuition tells me that Clipper is
;    creating a data area (as an assembler would, using the DB directive)
;    and the run time library reads this data area much like Foxbase does.
;    Except that Foxbase loads its data from a file. This theory would explain
;    how Foxbase could outrun Clipper last year. Normally, a true native mode
;    compiler would be extremely fast. Much faster than Clipper is now. How
;    much faster? Well, write an empty timing loop using FOR x = 1 to 100000.
;    Clipper     GWBASIC      Aztec C
;     107 secs     20 secs.     1 sec.
;    The CPU was 10Mhz 80286, and the C compiler didn't optimize the loop out.
;    As you can see, plenty of room for improvement!
;
;    For the record, my definition of a true native mode compiler is one in
;    which each line of source compiles into one or more op code/operand
;    machine instructions of the target CPU. In this case, Clipper would have
;    to generate Intel 8088, 80186, or 80386 op code/operands for each line of
;    dBASE code. Generating a "CALL  address" for each line would be 
;    permissible as often as necessary, but the PC (program counter) register
;    must be set at that address!
;
;    I'm sure to get many comments from this editorial. In spite of all of this,
;    Clipper is my compiler of choice. It works well and is flexible. Besides,
;    I have an interest in its success: my library.
;
;
;This dummy file strips out almost 50% of Clipper's run time stuff. A 1 line
;program will be about 76K! Now for the bad news. You can't perform any:
;  1) file operations
;  2) numeric operations
;
;The way to approach taking out stuff is to:
; 1) Create a real working program!
; 2) Check if your program has any of the following groups:
;    a) USE file.DBF      * almost always!
;    b) index operations  * ditto
;    c) memo files        * sometimes
;    d) Math functions    * usually some
;    e) Macro expansion   * hardly ever, if U write good code.
;    f) Network functions * sometimes
; 3) Assemble and link this file in with your program:
;    PLINK86 FI YOURPROG,DUMMY LIB CLIPPER
; 4) Run your program and note when it crashed. It will crash.
; 5) Find out what it was doing & comment out blocks in this file that
;    seem to relate to the crash. Look at the groups in this file. Decide 
;    what your program needs to run. Comment out any code in here that seems
;    to deal with what your program was trying to do.
; 6) Go to line 4. Keep doing this until your whole program runs correctly.
;
;You may notice that you had to leave most of the code alone, but hey, play
;around and see what you can do. I didn't go hog wild and strip Clipper
;clean, so I could have missed some stuff that isn't needed often.
;The way to poke around in Clipper is to use PLINK to create a link map.
;You do this by specifying the command to plink:
;   MAP=MYPROG.MAP M
; EX: PLINK86 FI MYPROG MAP=MYPROG M LIB CLIPPER
;
;The link map will show you the groups Nantucket put everything in. What I
;did was to look at the groups from the link map and then decide what I could
;take out. Now, in order to have the linker not link any of the group, you 
;have to take every symbol out of the group. (Note: compare a link map
;to the groups in this file.) This means the data section too.
;
;Unfortunately, PLINK86 truncates the symbol names, so You can't be sure that
;you specified the entire symbol name. So use MS-LINK or the linker provided
;with MS-DOS. Use the /M switch at the command line and a full symbol list
;will be generated with the symbols in alphabetical order and in address order.
;
;EX: LINK MYPROG,,MYPROG.MAP,CLIPPER /M
;
;Now, using your multi-window editor (I like Qedit) flip back and forth and
;make sure that you specify the entire symbol name.
;
;Again, I do not take any responsibility if you do this and cause your computer 
;to crash or what ever might happen. This is merely information I'm giving away 
;in hopes of making Nantucket repent its ways! Properly, Nantucket should 
;provide a compile switch to tell the linker not to link the the whole library 
;in.
;
;I think that the FORCE III compiler (not available as of this writing) will
;solve the unrealistic memory problem and will be worth buying because it will
;be a true native mode compiler and will only link in needed code. I'm promised,
;that a real working program will run in only 1 to 2 K! I am a beta test site
;for the compiler and I will keep you posted as to its code generation!
;
;For those who are interested. I'm the author of the Boatswain's Mate
;Toolbox for Clipper. There are over 200 functions and other tools to make
;development easier. You may down load an evaluation version of the library
;on my BBS at (603) 352-7919. It does everything the real library does. But
;it makes your program display our copyright notice once before any of our
;functions are called.
;
;Have fun. If you discover a way to make Clipper programs smaller. You can
;upload the info to the BBS (above) and I will combine it with other
;information I receive from other Clipper power users!
;---------------------------------------------------------------------------
;                            MACROS
;---------------------------------------------------------------------------
segdef  macro   sname
&sname segment byte public 'CODE'
        endm

segend macro sname
&sname  ends
        endm

datasegdef macro dname
&dname segment public 'DATA'
        endm

datadef macro dname
public &dname
&dname db ?
        endm

;Declare a dummy symbol that only returns. This satisfies the linker.
procdef macro pname
public  &pname
&pname proc far
        ret
&pname endp
        endm

;------------------------------------
;          BASIC .DBF file stuff
;------------------------------------
segdef DB_TEXT
procdef DBFILTER 
procdef DBRELATION
procdef DBRSELECT
procdef INDEXEXT
procdef INDEXORD
procdef NETERR
procdef __ADD_FIELD
procdef __APPEND
procdef __BILGE
procdef __BOF
procdef __close_dbf
procdef __CLR_RELATE
procdef __C_DBF
procdef __DBF_CLOSE
procdef __DBF_COMMIT
procdef __DBF_COPY
procdef __DBF_CREATE
procdef __DBF_DIR
procdef __DBF_OPEN
procdef __DBF_SELECT
procdef __DBF_SET
procdef __DBF_STRUX
procdef __DBGO_FIlter
procdef __DBGO_RElate
procdef __DBXBAG
procdef __DB_BREAk
procdef __DB_GOTO
procdef __DB_INIT
procdef __DB_QUIT
procdef __DELETE
procdef __DEL_FLAG
procdef __DEL_MARk
procdef __EOF
procdef __FALIAS0
procdef __FALIAS1
procdef __FCOUNT
procdef __FIELDNAME
procdef __FLOCK
procdef __FOUND
procdef __FSELECT0
procdef __FSELECT1
procdef __GOTO
procdef __GOTO_BOTT
procdef __GOTO_DB
procdef __GOTO_TOP
procdef __JEOF
procdef __LASTREC
procdef __LOCATEP
procdef __LOCK
procdef __NETNAME
procdef __NEXT
procdef __NTX_ORDER
procdef __PACK
procdef __RECALL
procdef __RECNO
procdef __REWIND
procdef __SCOPE_Dec
procdef __SCOPE_Set
procdef __SEEK
procdef __SELECT
procdef __SET_FILter
procdef __SET_LOCate
procdef __SET_RELate
procdef __SKIP
procdef __UNLOCK
procdef __USED
procdef __XNXT
procdef __XPOPF
procdef __XPUSHF
procdef __XREW
procdef __ZAP
segend DB_TEXT

datasegdef _DATA
datadef _DBF_DELIm
datadef __DBF
datadef __DBF1_SEl
datadef __DBF2_SEl
datadef __DBF3_SEl
datadef __DBF_MODe
datadef __NETERR
datadef __SELECTIon
segend _DATA

;------------------------------------
;          Index file stuff
;------------------------------------
segdef DISKIO_TEXT
procdef __ntx_blks
procdef __ntx_end
procdef __ntx_grow
procdef __ntx_init
procdef __ntx_shrink
procdef __page_append
procdef __page_flush
procdef __page_read
procdef __page_update
procdef __page_write
segend DISKIO_TEXT

datasegdef _DATA
datadef __page_error
segend _DATA

;------------------------------------
;        More Index stuff
;------------------------------------
segdef NTX_TEXT
procdef _NTX_COLD
procdef _NTX_HOLD
procdef __GO_SELF
procdef __key_save
procdef __key_update
procdef __kill_ntx
procdef __locate_key
procdef __next_key
procdef __NTX_CLOSE
procdef __NTX_COMMIT
procdef __NTX_CREATE
procdef __NTX_KEY
procdef __NTX_OPEN
procdef __ntx_put_header
procdef __REINDEX
procdef __seek_db
segend NTX_TEXT

datasegdef _DATA
datadef _BUFF_SIZE
datadef __indext
segend _DATA

;------------------------------------
;          Math Operations
;------------------------------------
segdef DVC_TEXT
procdef __dvabs
procdef __dveq
procdef __dveqz
procdef __dvexp
procdef __dvge
procdef __dvgt
procdef __dvint
procdef __dvle
procdef __dvlog
procdef __dvlt
procdef __dvltz
procdef __dvmod
procdef __dvne
procdef __dvneg
procdef __dvpow
procdef __dvrnd
procdef __dvsqr
segend DVC_TEXT

;------------------------------------
;          More Math Operations
;------------------------------------
segdef _TEXT
procdef __BFIL2
procdef __BFILD
procdef __BFIS2
procdef __BFIST
procdef __BFUST
procdef __DCVT
procdef __EADDD
procdef __EADDS
procdef __EDIVD
procdef __EDIVDR
procdef __EDIVS
procdef __EDIVSR
procdef __ELDD
procdef __ELDS
procdef __ELDW
procdef __EMULD
procdef __EMULS
procdef __ESTD
procdef __ESTDP
procdef __ESTS
procdef __ESTSP
procdef __ESUBD
procdef __ESUBDR
procdef __ESUBS
procdef __ESUBSR
procdef __FABS
procdef __FADD
procdef __FADDD
procdef __FADDS
procdef __FCHS
procdef __FCMP
procdef __FCOMPP
procdef __FDIV
procdef __FDIVD 
procdef __FDIVDR
procdef __FDIVR
procdef __FDIVS
procdef __FDIVSR
procdef __FDUP
procdef __FLDD
procdef __FLDL
procdef __FLDS
procdef __FLDT
procdef __FLDW 
procdef __FLDZ
procdef __FMUL
procdef __FMULD
procdef __FMULS
procdef __FSTD
procdef __FSTDP
procdef __FSTS
procdef __FSTSP
procdef __FSTT
procdef __FSTTP
procdef __FSUB
procdef __FSUBD
procdef __FSUBDR
procdef __FSUBR
procdef __FSUBS
procdef __FSUBSR
procdef __FTOL
procdef __FTST
procdef __FXCH
procdef __FXCHQ
procdef __SADDD
procdef __SADDS
procdef __SDIVD
procdef __SDIVDR
procdef __SDIVS
procdef __SDIVSR
procdef __SLDD
procdef __SLDL
procdef __SLDS
procdef __SLDT 
procdef __SLDW
procdef __SMULD
procdef __SMULS
procdef __SSTD
procdef __SSTDP
procdef __SSTS
procdef __SSTSP
procdef __SSTT
procdef __SSTTP
procdef __SSUBD
procdef __SSUBDR
procdef __SSUBS
procdef __SSUBSR
segend _TEXT

datasegdef _DATA
datadef OPTAB
segend _DATA

;------------------------------------
;           Math Operations
;------------------------------------
segdef _TEXT_CODE
procdef _MATHERR
procdef __ATON
procdef __DVADD
procdef __DVDIV
procdef __DVFMA
procdef __DVFMI
procdef __DVFML
procdef __DVFMQ
procdef __DVINIT
procdef __DVMUL
procdef __DVSUB
procdef __DVTOI
procdef __DVTOL
procdef __DVTOQ
procdef __LNTOA
procdef __NTOA
segend _TEXT_CODE

;------------------------------------
;          '&' Macro routines
;------------------------------------
segdef MACRO_TEXT
procdef __mac_execute
procdef __mac_exec_aug
procdef __mac_expand
procdef __mac_init
procdef __mac_release
procdef __mac_valid
procdef __TYPE
procdef __WAVE
procdef __WAVEA
procdef __WAVEL
procdef __WAVEP
procdef __WSET
procdef __WSYMBOL
procdef __XTYPE
segend MACRO_TEXT

;------------------------------------
;          Network Functions
;------------------------------------
segdef NET_TEXT
procdef __netalck
procdef __netaunl
procdef __netflck
procdef __netfunl
procdef __netrlck
procdef __netrunl
procdef __netwho
segend NET_TEXT

;------------------------------------
;        Low level file I/O
;------------------------------------
segdef FILESYS_TEXT
procdef __tclose
procdef __tcommit
procdef __tcreat
procdef __tend
procdef __thcount
procdef __tinit
procdef __tlock
procdef __tlseek
procdef __tmname
procdef __topen
procdef __tread
procdef __trename
procdef __tunlink
procdef __twrite
segend  FILESYS_TEXT

datasegdef _DATA
datadef __horror
datadef __terror
datadef __tversion
segend _DATA

;------------------------------------
;        Memo File stuff
;------------------------------------
segdef DBMEMO_TEXT
procdef __read_memo
procdef __sizeof_memo
procdef __write_memo
segend DBMEMO_TEXT

;------------------------------------
; Not sure, but dosen't seem to need it
;------------------------------------
segdef REEF_TEXT
procdef __alloc_reef
procdef __attach_reef
procdef __db_reef
procdef __detach_reef
procdef __form_reef
procdef __get_reef
procdef __kill_reef
procdef __purge_reef
procdef __read_reef
procdef __read_sdreef
procdef __REEF
procdef __reef_break
procdef __reef_init
procdef __reef_range
procdef __reef_size
procdef __trans_reef
procdef __write_reef
procdef __write_sdreef
segend REEF_TEXT

datasegdef _DATA
datadef __reef_scope
segend _DATA

;------------------------------------
;        Sorting routines
;------------------------------------
segdef SORTOF__TEXT
procdef __hold_stream
procdef __merge
procdef __quick_sort
procdef __sort_cleanup
segend SORTOF__TEXT

datasegdef _DATA
datadef __sfcount
datadef __sfields
datadef __slbase
datadef __srbase
datadef __srsize
segend _DATA

segdef _TEXT
procdef __setargv
segend _TEXT
         end
